@@ -69,6 +69,7 @@ class DedentNode(Node):
 
 
 class TemplateNode(HierNode):
+    """Represents the root Intermediate Representation node of a template."""
     class TemplateTail(Node):
         def py(self):
             yield self.line('template = kajiki.Template(template)')
@@ -241,6 +242,7 @@ class SwitchNode(HierNode):
 
     def py(self):
         yield self.line('local.__kj__.push_switch(%s)' % self.decl)
+        yield self.line('if False: pass')
 
     def __iter__(self):
         yield self
@@ -255,7 +257,7 @@ class CaseNode(HierNode):
         self.decl = decl
 
     def py(self):
-        yield self.line('if local.__kj__.case(%s):' % self.decl)
+        yield self.line('elif local.__kj__.case(%s):' % self.decl)
 
 
 class IfNode(HierNode):
