@@ -33,6 +33,27 @@ class _obj(object):
 
 
 class _Template(object):
+    """Base Class for all compiled Kajiki Templates.
+
+    All kajiki templates created from an ``ir.TemplateNode`` will
+    be subclasses of this class.
+
+    As the template body code runs inside ``__main__`` method of this
+    class, the instance of this class is always available as ``self``
+    inside the template code.
+
+    This class also makes available some global object inside the
+    template code itself:
+
+        - ``local`` which is the instance of the template
+        - ``defined`` which checks if the given variable is defined
+          inside the template scope.
+        - ``Markup`` which marks the passed object as markup code and
+          prevents escaping for its content.
+        - ``__kj__`` which is a special object used by generated code
+          providing features like keeping track of py:with stack or
+          or the gettext function used to translate text.
+    """
     __methods__ = ()
     loader = None
     base_globals = None
@@ -163,6 +184,8 @@ class _Template(object):
             attrs = attrs.items()
         if attrs is not None:
             for k, v in sorted(attrs):
+                if k in HTML_EMPTY_ATTRS and v in (True, False):
+                    v = k if v else None
                 if v is None:
                     continue
                 if mode.startswith('html') and k in HTML_EMPTY_ATTRS:
@@ -191,6 +214,25 @@ class _Template(object):
 
 
 def Template(ns):
+    """Creates a new ``_Template`` subclass from an entity with ``exposed`` functions.
+
+    Kajiki used classes as containers of the exposed functions for convenience,
+    but any object that can have the functions as attributes works.
+
+    To be a valid template the original entity must provide at least a ``__main__``
+    function::
+
+        class Example:
+            @kajiki.expose
+            def __main__():
+                yield 'Hi'
+
+        t = kajiki.Template(Example)
+        output = t().render()
+
+        print(output)
+        'Hi'
+    """
     dct = {}
     methods = dct['__methods__'] = []
     for name in dir(ns):
@@ -201,6 +243,12 @@ def Template(ns):
 
 
 def from_ir(ir_node):
+    """Creates a template class from Intermediate Representation TemplateNode.
+
+    This actually creates the class defined by the TemplateNode and returns
+    a subclass of it.
+    The returned class is a subclass of a `kajiki.template._Template`.
+    """
     py_lines = list(generate_python(ir_node))
     py_text = '\n'.join(map(str, py_lines))
     py_linenos = []
@@ -212,10 +260,8 @@ def from_ir(ir_node):
     dct = dict(kajiki=kajiki)
     try:
         exec(py_text, dct)
-    except (SyntaxError, IndentationError):  # pragma no cover
-        for i, line in enumerate(py_text.splitlines()):
-            print('%3d %s' % (i + 1, line))
-        raise
+    except (SyntaxError, IndentationError) as e:  # pragma no cover
+        raise KajikiSyntaxError(e.msg, py_text, e.filename, e.lineno, e.offset)
     tpl = dct['template']
     tpl.base_globals = dct
     tpl.py_text = py_text
@@ -316,3 +362,34 @@ else:
                             lnotab,
                             code.co_freevars,
                             code.co_cellvars)
+
+
+class KajikiSyntaxError(Exception):
+    def __init__(self, msg, source, filename, linen, coln):
+        super(KajikiSyntaxError, self).__init__(
+            '[%s:%s] %s\n%s' % (filename, linen, msg, self._get_source_snippet(source, linen))
+        )
+        self.filename = filename
+        self.linenum = linen
+        self.colnum = coln
+
+    def _get_source_snippet(self, source, linen):
+        SURROUNDING = 2
+        linen -= 1
+
+        parts = []
+        for i in range(SURROUNDING, 0, -1):
+            parts.append('\t     %s\n' % self._get_source_line(source, linen - i))
+        parts.append('\t --> %s\n' % self._get_source_line(source, linen))
+        for i in range(1, SURROUNDING+1):
+            parts.append('\t     %s\n' % self._get_source_line(source, linen + i))
+        return ''.join(parts)
+
+    def _get_source_line(self, source, linen):
+        if linen < 0:
+            return ''
+
+        try:
+            return source.splitlines()[linen]
+        except:
+            return ''
\ No newline at end of file
