@@ -6,6 +6,8 @@ import re
 from codecs import open
 from xml import sax
 from xml.dom import minidom as dom
+from xml.sax import SAXParseException
+
 from nine import IS_PYTHON2, basestring, str, iteritems, native_str
 
 if IS_PYTHON2:
@@ -27,6 +29,14 @@ impl = dom.getDOMImplementation(' ')
 
 def XMLTemplate(source=None, filename=None, mode=None, is_fragment=False,
                 encoding='utf-8', autoblocks=None, cdata_scripts=True):
+    """Given XML source code of a Kajiki Templates parses returns a Template class.
+
+    The source code is parsed to its DOM representation, which is then
+    expanded to separate directives from tags and then compiled to the
+    Intermediate Representation tree. The Intermediate Representation
+    is then processed to create the Python code which defined the template
+    class of which a new instance is returned.
+    """
     if source is None:
         with open(filename, encoding=encoding) as f:
             source = f.read()  # source is a unicode string
@@ -49,6 +59,11 @@ def annotate(gen):
 
 
 class _Compiler(object):
+    """Compiles a DOM tree into Intermediate Representation TemplateNode.
+
+    Intermediate Representation is a tree of nodes that represent
+    Python Code that should be generated to execute the template.
+    """
     def __init__(self, filename, doc, mode=None, is_fragment=False,
                  autoblocks=None, cdata_scripts=True):
         self.filename = filename
@@ -281,7 +296,12 @@ class _Compiler(object):
 
     @annotate
     def _compile_text(self, node):
-        tc = _TextCompiler(self.filename, node.data, node.lineno)
+        kwargs = {}
+        if node.parentNode and node.parentNode.tagName in HTML_CDATA_TAGS:
+            # script and style should always be untranslatable.
+            kwargs['node_type'] = ir.TextNode
+
+        tc = _TextCompiler(self.filename, node.data, node.lineno, **kwargs)
         for x in tc:
             yield x
 
@@ -302,9 +322,20 @@ class _Compiler(object):
 
     @annotate
     def _compile_switch(self, node):
-        # Filter out text nodes
-        body = [x for x in self._compile_nop(node)
-                if not isinstance(x, ir.TextNode)]
+        body = []
+
+        # Filter out empty text nodes and report unsupported nodes
+        for n in self._compile_nop(node):
+            if isinstance(n, ir.TextNode) and not n.text.strip():
+                continue
+            elif not isinstance(n, (ir.CaseNode, ir.ElseNode)):
+                raise XMLTemplateCompileError(
+                    'py:with directive can only contain py:case and py:else nodes '
+                    'and cannot be placed on a tag.',
+                    doc=self.doc, filename=self.filename, linen=node.lineno
+                )
+            body.append(n)
+
         yield ir.SwitchNode(node.getAttribute('test'), *body)
 
     @annotate
@@ -319,6 +350,15 @@ class _Compiler(object):
 
     @annotate
     def _compile_else(self, node):
+        if (getattr(node.parentNode, 'tagName', '') != 'py:nop' and
+                not node.parentNode.hasAttribute('py:switch') and
+                getattr(node.previousSibling, 'tagName', '') != 'py:if'):
+            raise XMLTemplateCompileError(
+                'py:else directive must be inside a py:switch or directly after py:if '
+                'without text or spaces in between',
+                doc=self.doc, filename=self.filename, linen=node.lineno
+            )
+
         yield ir.ElseNode(
             *list(self._compile_nop(node)))
 
@@ -332,6 +372,8 @@ class _Compiler(object):
 def make_text_node(text, guard=None):
     '''Return a TranslatableTextNode if the text is not empty,
     otherwise a regular TextNode.
+
+    This avoid spending the cost of translating empty nodes.
     '''
     if text.strip():
         return ir.TranslatableTextNode(text, guard)
@@ -413,6 +455,12 @@ class _TextCompiler(object):
 
 
 class _Parser(sax.ContentHandler):
+    """Parse an XML template into a Tree of DOM Nodes.
+
+    Nodes should then be passed to a `_Compiler` to be
+    converted into the intermediate representation and
+    then to Python Code.
+    """
     DTD = '<!DOCTYPE kajiki SYSTEM "kajiki.dtd">'
 
     def __init__(self, filename, source):
@@ -456,7 +504,16 @@ class _Parser(sax.ContentHandler):
         source.setEncoding(native_str('utf-8'))
         source.setByteStream(BytesIO(byts))
         source.setSystemId(self._filename)
-        parser.parse(source)
+
+        try:
+            parser.parse(source)
+        except SAXParseException as e:
+            exc = XMLTemplateParseError(e.getMessage(), self._source, self._filename,
+                                        e.getLineNumber(), e.getColumnNumber())
+            exc.__cause__ = None
+            raise exc
+
+        self._doc._source = self._source
         return self._doc
 
     # ContentHandler implementation
@@ -539,6 +596,24 @@ class _Parser(sax.ContentHandler):
 
 
 def expand(tree, parent=None):
+    """Expands directives attached to nodes into separate nodes.
+
+    This will convert all instances of::
+
+        <div py:if="check">
+        </div>
+
+    into::
+
+        <py:if test="check">
+            <div>
+            </div>
+        </py:if>
+
+    This ensures that whenever a template is processed there is no
+    different between the two formats as the Compiler will always
+    receive the latter.
+    """
     if isinstance(tree, dom.Document):
         expand(tree.firstChild, tree)
         return tree
@@ -574,3 +649,45 @@ def expand(tree, parent=None):
     for child in tree.childNodes:
         expand(child, tree)
     return tree
+
+
+class XMLTemplateError(Exception):
+    def __init__(self, msg, source, filename, linen, coln):
+        super(XMLTemplateError, self).__init__(
+            '[%s:%s] %s\n%s' % (filename, linen, msg, self._get_source_snippet(source, linen))
+        )
+        self.filename = filename
+        self.linenum = linen
+        self.colnum = coln
+
+    def _get_source_snippet(self, source, linen):
+        SURROUNDING = 2
+        linen -= 1
+
+        parts = []
+        for i in range(SURROUNDING, 0, -1):
+            parts.append('\t     %s\n' % self._get_source_line(source, linen - i))
+        parts.append('\t --> %s\n' % self._get_source_line(source, linen))
+        for i in range(1, SURROUNDING + 1):
+            parts.append('\t     %s\n' % self._get_source_line(source, linen + i))
+        return ''.join(parts)
+
+    def _get_source_line(self, source, linen):
+        if linen < 0:
+            return ''
+
+        try:
+            return source.splitlines()[linen]
+        except:
+            return ''
+
+
+class XMLTemplateCompileError(XMLTemplateError):
+    def __init__(self, msg, doc, filename, linen):
+        super(XMLTemplateCompileError, self).__init__(
+            msg, getattr(doc, '_source', ''), filename, linen, 0
+        )
+
+
+class XMLTemplateParseError(XMLTemplateError):
+    pass
